# -*- eval: (blacken-mode) -*-
import logging
import pytest
import numpy as np
from numpy.testing import assert_allclose

import usadelndsoc
import usadelndsoc.solver

usadelndsoc.logger.setLevel(logging.DEBUG)


def basic_setup(terminals=False, nx=15, ny=5):
    Delta = np.zeros((4, 4))
    Delta[:2, 2:] = np.eye(2)
    Delta[2:, :2] = np.eye(2)

    s = usadelndsoc.solver.Solver(nx=nx, ny=ny)
    s.Omega[...] = 1j * Delta
    s.alpha = 0.0
    s.Lx = 10
    s.Ly = 10

    if terminals:
        s.mask[0, :] = usadelndsoc.solver.MASK_TERMINAL
        s.mask[-1, :] = usadelndsoc.solver.MASK_TERMINAL

    return s


# s.solve(omega=150, maxiter=300, preconditioner="none")


@pytest.mark.slow
def test_solve_dos():
    s = basic_setup()
    E = np.linspace(-3, 3, 101) + 0.05j
    res = s.solve_many(omega=-1j * E)
    g = res.G[:, 10, 2, 0, 0]
    g_an = -1j * E / np.sqrt(1 - E**2)
    assert_allclose(g, g_an, rtol=1e-4)


def test_result_S_J0():
    s = basic_setup()
    s.Omega[:, :, 0, 3] = 0.5j
    s.Omega[:, :, 3, 0] = 0.5j
    res = s.solve_many(omega=0.5 + 0.8j)
    S = res.S

    vol = s.Lx * s.Ly / (s.shape[0] * s.shape[1]) * 4j

    assert_allclose(S[:, :, :2, :2], vol * res.G)
    assert_allclose(S[:, :, :2, 2:], vol * res.F)
    assert_allclose(S[:, :, 2:, :2], vol * res.Fc)
    assert_allclose(S[:, :, 2:, 2:], vol * res.Gc)

    # There are no gradients, hence no currents either
    J = res.J
    assert_allclose(J, 0, atol=1e-5)
